@@ -20,7 +20,8 @@ import torch
 from numpy.typing import ArrayLike
 
 from pyqtorch.converters.store_ops import ops_cache, store_operation
-from pyqtorch.core.utils import OPERATIONS_DICT, _apply_gate
+from pyqtorch.core.utils import _apply_gate
+from pyqtorch.matrices import DEFAULT_MATRIX_DTYPE, OPERATIONS_DICT
 
 
 def get_parametrized_matrix_for_operation(operation_type: str, theta: torch.Tensor) -> torch.Tensor:
@@ -40,20 +41,24 @@ def get_parametrized_matrix_for_operation(operation_type: str, theta: torch.Tens
 
 
 def create_controlled_matrix_from_operation(
-    operation_matrix: torch.Tensor,
+    operation_matrix: torch.Tensor, n_control_qubits: int = 1
 ) -> torch.Tensor:
-    """Method which takes a 2x2 torch.Tensor and transforms it into a Controlled Operation Gate
+    """Method which takes a torch.Tensor and transforms it into a Controlled Operation Gate
 
     Args:
 
-        operation_matrix (torch.Tensor): the type of operation which should be performed (RX,RY,RZ)
+        operation_matrix: (torch.Tensor): the type of operation which should be
+        performed (RX,RY,RZ,SWAP)
+        n_control_qubits: (int): The number of control qubits used
 
     Returns:
 
         torch.Tensor: the resulting controlled gate populated by operation_matrix
     """
     mat_size = len(operation_matrix)
-    controlled_mat: torch.Tensor = torch.eye(2 * mat_size, dtype=torch.cdouble)
+    controlled_mat: torch.Tensor = torch.eye(
+        2**n_control_qubits * mat_size, dtype=DEFAULT_MATRIX_DTYPE
+    )
     controlled_mat[-mat_size:, -mat_size:] = operation_matrix
     return controlled_mat
 
@@ -134,9 +139,9 @@ def RZZ(theta: torch.Tensor, state: torch.Tensor, qubits: ArrayLike, N_qubits: i
         store_operation("RZZ", qubits, param=theta)
 
     dev = state.device
-    mat = torch.diag(torch.tensor([1, -1, -1, 1], dtype=torch.cdouble).to(dev))
+    mat = torch.diag(torch.tensor([1, -1, -1, 1], dtype=DEFAULT_MATRIX_DTYPE).to(dev))
     mat = 1j * torch.sin(theta / 2) * mat + torch.cos(theta / 2) * torch.eye(
-        4, dtype=torch.cdouble
+        4, dtype=DEFAULT_MATRIX_DTYPE
     ).to(dev)
     return _apply_gate(state, torch.diag(mat), qubits, N_qubits)
 
@@ -173,14 +178,16 @@ def U(
     t_plus = torch.exp(-1j * (phi + omega) / 2)
     t_minus = torch.exp(-1j * (phi - omega) / 2)
     mat = (
-        torch.tensor([[1, 0], [0, 0]], dtype=torch.cdouble).to(dev) * torch.cos(theta / 2) * t_plus
-        - torch.tensor([[0, 1], [0, 0]], dtype=torch.cdouble).to(dev)
+        torch.tensor([[1, 0], [0, 0]], dtype=DEFAULT_MATRIX_DTYPE).to(dev)
+        * torch.cos(theta / 2)
+        * t_plus
+        - torch.tensor([[0, 1], [0, 0]], dtype=DEFAULT_MATRIX_DTYPE).to(dev)
         * torch.sin(theta / 2)
         * torch.conj(t_minus)
-        + torch.tensor([[0, 0], [1, 0]], dtype=torch.cdouble).to(dev)
+        + torch.tensor([[0, 0], [1, 0]], dtype=DEFAULT_MATRIX_DTYPE).to(dev)
         * torch.sin(theta / 2)
         * t_minus
-        + torch.tensor([[0, 0], [0, 1]], dtype=torch.cdouble).to(dev)
+        + torch.tensor([[0, 0], [0, 1]], dtype=DEFAULT_MATRIX_DTYPE).to(dev)
         * torch.cos(theta / 2)
         * torch.conj(t_plus)
     )
@@ -410,8 +417,8 @@ def S(state: torch.Tensor, qubits: ArrayLike, N_qubits: int) -> torch.Tensor:
     return _apply_gate(state, mat, qubits, N_qubits)
 
 
-def Sdagger(state: torch.Tensor, qubits: ArrayLike, N_qubits: int) -> torch.Tensor:
-    """Sdagger single-qubit gate
+def SDagger(state: torch.Tensor, qubits: ArrayLike, N_qubits: int) -> torch.Tensor:
+    """SDagger single-qubit gate
 
     Args:
         state (torch.Tensor): the input quantum state, of shape `(N_0, N_1,..., N_N, batch_size)`
@@ -423,7 +430,7 @@ def Sdagger(state: torch.Tensor, qubits: ArrayLike, N_qubits: int) -> torch.Tens
     """
 
     if ops_cache.enabled:
-        store_operation("Sdagger", qubits)
+        store_operation("SDagger", qubits)
 
     dev = state.device
     mat = OPERATIONS_DICT["SDAGGER"].to(dev)
@@ -450,6 +457,26 @@ def T(state: torch.Tensor, qubits: ArrayLike, N_qubits: int) -> torch.Tensor:
     return _apply_gate(state, mat, qubits, N_qubits)
 
 
+def N(state: torch.Tensor, qubits: ArrayLike, N_qubits: int) -> torch.Tensor:
+    """N (projector) single-qubit gate (I-Z)/2
+
+    Args:
+        state (torch.Tensor): the input quantum state, of shape `(N_0, N_1,..., N_N, batch_size)`
+        qubits (ArrayLike): list of qubit indices where the gate will operate
+        N_qubits (int): the number of qubits in the system
+
+    Returns:
+        torch.Tensor: the resulting state after applying the gate
+    """
+
+    if ops_cache.enabled:
+        store_operation("N", qubits)
+
+    dev = state.device
+    mat = OPERATIONS_DICT["N"].to(dev)
+    return _apply_gate(state, mat, qubits, N_qubits)
+
+
 def SWAP(state: torch.Tensor, qubits: ArrayLike, N_qubits: int) -> torch.Tensor:
     """SWAP 2-qubit gate
 
@@ -495,7 +522,7 @@ def CPHASE(
             [0, 0, 1, 0],
             [0, 0, 0, torch.exp(torch.tensor(1j * theta))],
         ],
-        dtype=torch.cdouble,
+        dtype=DEFAULT_MATRIX_DTYPE,
     ).to(dev)
     return _apply_gate(state, mat, qubits, N_qubits)
 
@@ -588,8 +615,8 @@ def diagonalize(H: torch.Tensor) -> Tuple[torch.Tensor, Optional[torch.Tensor]]:
     else:
         if is_real(H):
             eig_values, eig_vectors = torch.linalg.eigh(H.real)
-            eig_values = eig_values.to(torch.cdouble)
-            eig_vectors = eig_vectors.to(torch.cdouble)
+            eig_values = eig_values.to(DEFAULT_MATRIX_DTYPE)
+            eig_vectors = eig_vectors.to(DEFAULT_MATRIX_DTYPE)
         else:
             eig_values, eig_vectors = torch.linalg.eigh(H)
 
@@ -625,7 +652,7 @@ def hamiltonian_evolution_eig(
     batch_size_s = state.size()[-1]
     batch_size_t = len(t)
 
-    t_evo = torch.zeros(batch_size_s).to(torch.cdouble)
+    t_evo = torch.zeros(batch_size_s).to(DEFAULT_MATRIX_DTYPE)
 
     if batch_size_t >= batch_size_s:
         t_evo = t[:batch_size_s]
